@@ -125,7 +125,7 @@ class VariableInterface(object):
     def select(self, *fields, **kw):
         pass
 
-    def ZipVars(self, *argvVarNames):
+    def ZipVars(self, *argvVarNames, **kw):
         pass
 
     def uncompatible(self):
@@ -313,11 +313,16 @@ class Variable(VariableInterface):
         """getRemoteInfo by datavars"""
         return self.parent.getRemoteInfo(envFile)
 
-    def ZipVars(self, *argvVarNames):
+    def ZipVars(self, *argvVarNames, **kw):
         """
         Get zipped values of variables specified by list 'argvVarNames'
         """
-        return zip(*map(self.Get, argvVarNames))
+        if "humanreadable" in kw:
+            hr = kw['humanreadable']
+            return zip(*map(lambda x:self.Get(x, humanreadable=hr),
+                            argvVarNames))
+        else:
+            return zip(*map(self.Get, argvVarNames))
 
     def _get_format_value(self):
         return self._value_formatter.format(
@@ -1067,17 +1072,25 @@ class SimpleDataVars(object):
             return map(getList(':'), value.split(','))
         return fixEmpty(value).strip("'").strip('"')
 
-    def ZipVars(self, *argvVarNames):
+    def ZipVars(self, *argvVarNames, **kw):
         """
         Get zipped values of variables specified by list 'argvVarNames'
         """
-        return zip(*map(self.Get, argvVarNames))
+        if "humanreadable" in kw:
+            hr = kw['humanreadable']
+            return zip(*map(lambda x:self.Get(x, humanreadable=hr),
+                            argvVarNames))
+        else:
+            return zip(*map(self.Get, argvVarNames))
 
     def select(self, *fields, **kw):
         """
         Новая функция выборки из табличных переменных
+
         :param fields: необходимые поля
         :param kw: аргумены выборки
+        :param limit: количество возвращаемых элементов
+        :param zip_vars: функция объединения переменных
         :return:
         """
         keywords = ["limit", "zip_vars"]
@@ -1096,6 +1109,7 @@ class SimpleDataVars(object):
                 'ne': lambda x, y: lambda z: z[x] != y,
                 'in': lambda x, y: lambda z: z[x] in y,
                 'not_in': lambda x, y: lambda z: z[x] not in y,
+                'startswith': lambda x, y: lambda z: z[x].startswith(y)
             }
             filter_chain.append(func_maps[funcname](len(variables) - 1, kw[k]))
         l = len(fields)
@@ -1131,11 +1145,14 @@ class SimpleDataVars(object):
 
     def Select(self, selField, where="os_disk_dev", eq=None, ne=None,
                _in=None, _notin=None, like=None, notlike=None, func=None,
+               _if=None,
                sort=None, sortkey=None, limit=None, zipVars=None):
         """Select value from table variables"""
         if zipVars is None:
             zipVars = self.ZipVars
-        if func:
+        if func or _if:
+            if _if:
+                func = _if
             if func.func_code.co_argcount > 1:
                 filterFunc = lambda x: func(*x[:func.func_code.co_argcount])
             else:
